@@ -17,6 +17,7 @@
 #include <sys/mount.h>
 #include <sys/prctl.h>
 #include <sys/stat.h>
+#include <sys/statvfs.h>
 #include <sys/syscall.h>
 #include <sys/types.h>
 #include <time.h>
@@ -98,6 +99,7 @@ void join_namespaces(pid_t pid);
 void join_end();
 void log_ids(const char *func, int line);
 bool path_exists(char *path);
+unsigned long path_mount_flags(char *path);
 void path_split(char *path, char **dir, char **base);
 void sem_timedwait_relative(sem_t *sem, int timeout);
 void setup_namespaces(struct container *c);
@@ -193,23 +195,24 @@ void enter_udss(struct container *c)
    }
    // Container /home.
    if (!c->private_home) {
-      char *oldhome, *newhome;
+      char *newhome;
       // Mount tmpfs on guest /home because guest root is read-only
       tmpfs_mount("/home", c->newroot, "size=4m");
       // Bind-mount user's home directory at /home/$USER. The main use case is
       // dotfiles.
-      oldhome = getenv("HOME");
-      Tf (oldhome != NULL, "cannot find home directory: $HOME not set");
+      Tf (c->old_home != NULL, "cannot find home directory: is $HOME set?");
       newhome = cat("/home/", getenv("USER"));
       Z_ (mkdir(cat(c->newroot, newhome), 0755));
-      bind_mount(oldhome, newhome, c->newroot, BD_REQUIRED, 0);
+      bind_mount(c->old_home, newhome, c->newroot, BD_REQUIRED, 0);
    }
-   // Bind-mount /usr/bin/ch-ssh if it exists.
-   if (path_exists(cat(c->newroot, "/usr/bin/ch-ssh"))) {
+   // Container /usr/bin/ch-ssh.
+   if (c->ch_ssh) {
       char chrun_file[PATH_CHARS];
       int len = readlink("/proc/self/exe", chrun_file, PATH_CHARS);
       T_ (len >= 0);
-      chrun_file[ len<PATH_CHARS ? len : PATH_CHARS-1 ] = 0; // terminate; #314
+      Te (path_exists(cat(c->newroot, "/usr/bin/ch-ssh")),
+          "--ch-ssh: /usr/bin/ch-ssh not in image");
+      chrun_file[ len<PATH_CHARS ? len : PATH_CHARS-1 ] = 0; // terminate; #315
       bind_mount(cat(dirname(chrun_file), "/ch-ssh"), "/usr/bin/ch-ssh",
                  c->newroot, BD_REQUIRED, 0);
    }
@@ -225,7 +228,11 @@ void enter_udss(struct container *c)
 
    // Re-mount new root read-only unless --write or already read-only.
    if (!c->writable && !(access(c->newroot, W_OK) == -1 && errno == EROFS)) {
-      Zf (mount(NULL, c->newroot, NULL, MS_REMOUNT|MS_BIND|MS_RDONLY, NULL),
+      unsigned long flags =   path_mount_flags(c->newroot)
+                            | MS_REMOUNT  // Re-mount ...
+                            | MS_BIND     // only this mount point ...
+                            | MS_RDONLY;  // read-only.
+      Zf (mount(NULL, c->newroot, NULL, flags, NULL),
           "can't re-mount image read-only (is it on NFS?)");
    }
    // Pivot into the new root. Use /dev because it's available even in
@@ -405,6 +412,38 @@ bool path_exists(char *path)
    return false;
 }
 
+/* Return the mount flags of the file system containing path, suitable for
+   passing to mount(2).
+
+   This is messy because, the flags we get from statvfs(3) are ST_* while the
+   flags needed by mount(2) are MS_*. My glibc has a comment in bits/statvfs.h
+   that the ST_* "should be kept in sync with" the MS_* flags, and the values
+   do seem to match, but there are additional undocumented flags in there.
+   Also, the kernel contains a test "unprivileged-remount-test.c" that
+   manually translates the flags. Thus, I wasn't comfortable simply passing
+   the output of statvfs(3) to mount(2). */
+unsigned long path_mount_flags(char *path)
+{
+   struct statvfs sv;
+   unsigned long known_flags =   ST_MANDLOCK   | ST_NOATIME  | ST_NODEV
+                               | ST_NODIRATIME | ST_NOEXEC   | ST_NOSUID
+                               | ST_RDONLY     | ST_RELATIME | ST_SYNCHRONOUS;
+
+   Z_ (statvfs(path, &sv));
+   Ze (sv.f_flag & ~known_flags, "unknown mount flags: 0x%lx %s",
+       sv.f_flag & ~known_flags, path);
+
+   return   (sv.f_flag & ST_MANDLOCK    ? MS_MANDLOCK    : 0)
+          | (sv.f_flag & ST_NOATIME     ? MS_NOATIME     : 0)
+          | (sv.f_flag & ST_NODEV       ? MS_NODEV       : 0)
+          | (sv.f_flag & ST_NODIRATIME  ? MS_NODIRATIME  : 0)
+          | (sv.f_flag & ST_NOEXEC      ? MS_NOEXEC      : 0)
+          | (sv.f_flag & ST_NOSUID      ? MS_NOSUID      : 0)
+          | (sv.f_flag & ST_RDONLY      ? MS_RDONLY      : 0)
+          | (sv.f_flag & ST_RELATIME    ? MS_RELATIME    : 0)
+          | (sv.f_flag & ST_SYNCHRONOUS ? MS_SYNCHRONOUS : 0);
+}
+
 /* Split path into dirname and basename. */
 void path_split(char *path, char **dir, char **base)
 {
@@ -533,6 +572,24 @@ void setup_passwd(struct container *c)
    Z_ (unlink(path));
 }
 
+/* Split string str at first instance of delimiter del. Set *a to the part
+   before del, and *b to the part after. Both can be empty; if no token is
+   present, set both to NULL. Unlike strsep(3), str is unchanged; *a and *b
+   point into a new buffer allocated with malloc(3). This has two
+   implications: (1) the caller must free(3) *a but not *b, and (2) the parts
+   can be rejoined by setting *(*b-1) to del. The point here is to provide an
+   easier wrapper for strsep(3). */
+void split(char **a, char **b, char *str, char del)
+{
+   char delstr[2] = { del, 0 };
+   T_ (str != NULL);
+   str = strdup(str);
+   *b = str;
+   *a = strsep(b, delstr);
+   if (*b == NULL)
+      *a = NULL;
+}
+
 /* Mount a tmpfs at the given path. */
 void tmpfs_mount(char *dst, char *newroot, char *data)
 {
