@@ -65,6 +65,7 @@ enum bind_dep {
 
 struct container {
    struct bind *binds;
+   bool ch_ssh;         // bind /usr/bin/ch-ssh?
    gid_t container_gid;
    uid_t container_uid;
    char *newroot;
@@ -74,6 +75,7 @@ struct container {
    char *join_tag;      // identifier for synchronized join
    bool private_home;
    bool private_tmp;
+   char *old_home;      // host path to user's home directory (i.e. $HOME)
    bool writable;
 };
 
@@ -88,4 +90,5 @@ extern int verbose;
 void containerize(struct container *c);
 void msg(int level, char *file, int line, int errno_, char *fmt, ...);
 void run_user_command(char *argv[], char *initial_dir);
+void split(char **a, char **b, char *str, char del);
 void version(void);
