@@ -26,6 +26,14 @@ case $TARBALL in
         tar xf charliecloud-*.tar.gz
         cd charliecloud-*
         ;;
+    export-bats)
+        (cd doc-src && make)
+        make export-bats
+        mv charliecloud-*.tar.gz "$PREFIX"
+        cd "$PREFIX"
+        tar xf charliecloud-*.tar.gz
+        cd charliecloud-*
+        ;;
     archive)
         # The Travis image already has Bats installed.
         git archive HEAD --prefix=charliecloud/ -o "$PREFIX/charliecloud.tar"
@@ -35,34 +43,31 @@ case $TARBALL in
         ;;
 esac
 
-if [[ $PKG_BUILD ]]; then
-    for i in packaging/*/travis.sh; do $i; done
-    # FIXME: If we continue with the rest of the tests after building the
-    # packages, they hang in "make test-all", I believe in test "ch-build
-    # python3" but I have not been able to verify this.
-    exit
-fi
-
 make
 bin/ch-run --version
 
 if [[ $INSTALL ]]; then
     sudo make install PREFIX="$PREFIX"
-    cd "$PREFIX/share/doc/charliecloud"
+    cd "$PREFIX/libexec/charliecloud"
 fi
 
 cd test
 
 make where-bats
-make test
+make test-build
 
-# To test without Docker, move the binary out of the way.
-DOCKER=$(command -v docker)
-sudo mv "$DOCKER" "$DOCKER.tmp"
+if [[ $SUDO_RM_AFTER_BUILD ]]; then
+    sudo rm /etc/sudoers.d/travis
+fi
+if [[ $SUDO_AVOID_AFTER_BUILD ]]; then
+    export CH_TEST_DONT_SUDO=yes
+fi
+if ( sudo -v ); then
+    echo "have sudo"
+else
+    echo "don't have sudo"
+fi
+echo "\$CH_TEST_DONT_SUDO=$CH_TEST_DONT_SUDO"
 
-make test
-
-# For Travis, this isn't really necessary, since the VM will go away
-# immediately after this script exits. However, restore the binary to enable
-# testing this script in other environments.
-sudo mv "$DOCKER.tmp" "$DOCKER"
+make test-run
+make test-test
